import xlwings as xw
import numpy as np
from numba import jit

@jit(nopython=True)
def Newmark_nb(m, k, c, dt, f, x0 , xp0):
    """ 
    Integration with Newmark's method.
    
    See Chopra, "Dynamics of structures", Ch. 5.4
    """
     
    N = len(f)
    x = np.zeros(N)
    xp = np.zeros(N)
    xpp = np.zeros(N)
            
    # parametri Newmark
    gamma = 1./2.
    beta  = 1./4. # select 1/4. for Average acceleration method, 1/6. for linear acceleration method.
    
    # condizioni iniziali
    x[0] = x0
    xp[0] = xp0
    xpp[0] = (f[0] - c*xp0 - k*x0) / m
    # 
    k_hat = k + gamma/(beta*dt)*c + 1./(beta*dt**2)*m
    a = 1./(beta*dt)*m + gamma/beta*c
    b = 1./(2*beta)*m + dt*(gamma/(2*beta)-1)*c
            
    for j in range(0,N-1):
        # 
        df = (f[j+1]-f[j]) + a*xp[j] + b*xpp[j]
        dx = df/k_hat
        dxp = gamma/(beta*dt)*dx - gamma/beta*xp[j] + dt*(1-gamma/(2*beta))*xpp[j]
        dxpp = 1./(beta*dt**2)*dx - 1./(beta*dt)*xp[j] - 1./(2*beta)*xpp[j]
        # compute the values at step j+1
        x[j+1] = x[j] + dx
        xp[j+1] = xp[j] + dxp
        xpp[j+1] = xpp[j] + dxpp
   
    return x, xp, xpp

@xw.sub  # only required if you want to import it or run it via UDF Server
def main():
    wb = xw.Book.caller()
    sheet = wb.sheets[0]
    if sheet["A1"].value == "Hello xlwings!":
        sheet["A1"].value = "Bye xlwings!"
    else:
        sheet["A1"].value = "Hello xlwings!"


@xw.func
def hello(name):
    return "hello {0}".format(name)

@xw.func
@xw.arg('f', np.array, ndim=1)
def risposta(f, dt, T1):
    m = 1.
    w1 = 2*np.pi/T1
    k = m*w1**2
    v = 0.05
    c = m*2*v*w1
    x0 = 0.
    xp0 = 0.
    x, xp, xpp = Newmark_nb(m, k, c, dt, f, x0 , xp0)
    return x.reshape((len(x),1))
    
@xw.func
@xw.arg('f', np.array, ndim=1)
def massimo(f, dt, T1):
    m = 1.
    w1 = 2*np.pi/T1
    k = m*w1**2
    v = 0.05
    c = m*2*v*w1
    x0 = 0.
    xp0 = 0.
    x, xp, xpp = Newmark_nb(m, k, c, dt, f, x0 , xp0)
    return np.max(np.abs(x))

if __name__ == "__main__":
    xw.Book("prova.xlsm").set_mock_caller()
    main()
